from os.path import expanduser
import os
import json
import editor
import re

class ConfigManager(object):
	"Manages Client connection details"
	def __init__(self):
		pass

	@classmethod
	def prepare_config_dir(cls):
		home  = os.path.join(expanduser("~"),  ".jira")
		if not os.path.exists(home):
			os.makedirs(home)
		return home

	@classmethod
	def get_config_file_name(cls):
		home = cls.prepare_config_dir()
		return os.path.join(home, "config.json")

	@classmethod
	def save_details(cls, host, user, password):
		config_file = cls.get_config_file_name();
		details = {'user': user, 'host':host, 'password':password}
		save_json_to_file(details, config_file)

	@classmethod
	def _check_config(cls, json_obj):
		for key in ['user', 'host', 'password']:
			if key not in json_obj:
				return False
		return True

	@classmethod	
	def get_details(cls):
		config_file = cls.get_config_file_name();
		try:
			json_obj = get_json_from_file(config_file)
			if not cls._check_config(json_obj):	
				raise InvalidConfiguration("Invalid configuration")
		except IOError:
			raise InvalidConfiguration("Configuration file doesnt exists")
		except ValueError:
			raise InvalidConfiguration("Invalid configuration")
		return json_obj

issue_template = """##Summary##
 Write Summary Here
##End##
##Description##
 Write Description Here
##End##
"""
issue_regex = r'##Summary##(?P<summary>(.|\n)*)\n##End##\n##Description##(?P<desc>(.|\n)*)\n##End##'
comment_template = """
****** Comment ******
 Write Comment Here
****** End ******
"""
class EditorMode(object):
	def __init__(self, type):
		self.type = type

	def open(self):
		template = issue_template if self.type == 'issue' else comment_template
		self.value = editor.edit(contents=template)

	def _parse_issue_template(self):
		text = self.value
		reg_search = re.search(issue_regex, text)
		data = reg_search.groupdict()
		summary = data['summary'];
		description = data['desc'];
		return {'summary':summary, 'description':description}

	def parse(self):
		return self._parse_issue_template() if self.type == 'issue' else self._parse_comment_template()

	def get(self):
		return self.parse(self.value)


class InvalidConfiguration(Exception):
	def __init__(self, value):
		self.value = value
	def __str__(self):
		return repr(self.value)

def ask_for_confirmation(text):
	while True:
		res = str(input(text))
		if res.lower() not in ['y', 'n']:
			print 'Please try with a valid response'
			continue
		else:
			return res.lower() == 'y'



def save_json_to_file(json_obj, file):
	with open(file, 'w') as config:
		config.write(json.dumps(json_obj))


def get_json_from_file(file):
	with open(file, 'r') as config:
		return json.loads(config.read())



import click
from utils import ConfigManager, InvalidConfiguration, EditorMode, ask_for_confirmation
from jira_service import Jira, IssueMapper
import editor

@click.group()
def cli():
	"""Simple Jira Command Line Client"""
	pass

@cli.command()
@click.option("--host", prompt=True)
@click.option("--user", prompt=True)
@click.option("--password", prompt=True, hide_input=True,
              confirmation_prompt=True)
def configure(host, user, password):
	"""Configure client with HOST, USER, PASSWORD"""
	ConfigManager.save_details(host, user, password)

@cli.command()
@click.argument("issue_key")
@click.option("--full", "-f", is_flag=True)
def issue(issue_key, full):
	"""Gets issue by issue key"""
	jra = Jira.get_jira_service();	# Check for exception and ask user to configure
	mapper = IssueMapper(jra.get_issue(issue_key))
	rep = mapper.get_long_rep() if full else mapper.get_short_rep()
	print rep

@cli.command("ls")
@click.option("--project", "-p", help="Project name")
@click.option("--assignee", "-s", help ="Filter by assignee")
@click.option("--all", "-a", is_flag=True, help="Get all issues")
def list(project, assignee, all):
	"""Lists Issues"""
	jra = Jira.get_jira_service();
	issues = jra.get_issue_for_user(assignee, project, all) # shift list representation logic to IssueMapper
	issue_reps = map(lambda x:IssueMapper(x).get_short_rep(), issues)
	map(printf, issue_reps)

@cli.command("create")
@click.argument("project")
@click.argument("type")
@click.option("--summary", "-s", help="Summary")
@click.option("--desc", "-d" , help="Description", default = "")
@click.option("--edit_mode", "-e" , is_flag = True)
def create(project, type,summary, desc, edit_mode):
	"""Creates issue under a project"""
	if edit_mode:
		interface =EditorMode('issue')
		interface.open()
		data = interface.parse()
		summary = data['summary']	
		description = data['description']
		if not ask_for_confirmation('Are you sure you want to create issue(Y/N): '):
			print 'Operation Aborted'
			return None
	jra = Jira.get_jira_service();	
	issue = jra.create_issue(project, summary, description, type)
	print IssueMapper(issue).get_long_rep()

@cli.command("comment")
@click.argument("issue_key")
@click.argument("comment")
def create(issue_key, comment):
	"""Creates issue under a project"""
	jra = Jira.get_jira_service();
	print jra.add_comment(issue_key, comment)

@cli.command("edit")
@click.argument("issue_key")
@click.option("--summary", "-s", help="Summary")
@click.option("--desc", "-d" , help="Description", default = "")
@click.option("--edit_mode", "-e" , is_flag = True)
def create(issue_key, comment):
	"""Creates issue under a project"""
	if edit_mode:
		interface =EditorMode('issue')
		interface.open()
		data = interface.parse()
		summary = data['summary']	
		description = data['description']
		if not ask_for_confirmation('Are you sure you want to create issue(Y/N): '):
			print 'Operation Aborted'
			return None
	jra = Jira.get_jira_service();
	


def printf(str):
	print str

if __name__ == '__main__':
	try:
		cli()
	except InvalidConfiguration, e:
		print "Error: " + str(e)
		print "To reconfigure client run - \n'jira configure'"
		print cli.get_help(click.Context(cli))


from jira import JIRA as jra
from utils import ConfigManager

class Jira(object):

	def __init__(self, host, user, password):
		self.host = host
		self.user = user
		self.password = password
		self.jra = jra(host, basic_auth=(user, password))

	@classmethod
	def get_jira_service(cls):
		config_details = ConfigManager.get_details();
		jira = Jira(config_details['host'], config_details['user'], config_details['password'])
		return jira

	def get_issue(self, issue_key):
		issue = self.jra.issue(issue_key)
		return issue

	def create_issue(self, project, title, desc, type ):
		issue_detail = {
			'project': {'key':project},
			'summary': title,
			'description': desc,
			'issuetype':{'name':type}
		}
		return self.jra.create_issue(fields = issue_detail)

	def get_issue_for_user(self, user, project, all):
		user = self.user if user is None else user 
		jql = "assignee = %s" % user
		jql = jql + " and project = %s"%project if project else jql
		jql = jql + " and resolution = Unresolved" if not all else jql
		print jql
		return self.jra.search_issues(jql)

	def add_comment(self, issue_key, comment):
		comment = self.jra.add_comment(issue_key, comment);

class IssueMapper(object):
	def __init__(self, issue):
		self.issue = issue

	def get_short_rep(self):

		return "[%s] - %s" % (self.issue.fields.issuetype, self.issue.fields.summary)

	def get_long_rep(self):
		desc = self.issue.fields.description
		desc = desc.strip() if desc is not None else ""
		return "[Type] - %s\n[Title] - %s\n[Assignee] - %s\t[Reporter] - %s\n[Description]\n%s" % (self.issue.fields.issuetype, self.issue.fields.summary.strip(), self.issue.fields.assignee, self.issue.fields.reporter, desc)


from distutils.core import setup
setup(
  name = 'pjira',
  packages = ['pjira'], # this must be the same as the name above
  version = '0.1',
  description = 'A simple Atlassian jira cli client written in python',
  author = 'Shubham Sharma',
  author_email = 'shubham.sha12@gmail.com',
  url = 'https://github.com/gabber12/python-jira', # use the URL to the github repo
  download_url = 'https://github.com/gabber12/python-jira/archive/0.1.1.tar.gz', # I'll explain this in a second
  keywords = ['client', 'cli', 'jira'], # arbitrary keywords
  classifiers = [],
)
